# rust
[quick reference guide for the Pythonista in the process of becoming a Rustacean](https://github.com/rochacbruno/py2rs)
[Learn Rust in Y Minutes](https://learnxinyminutes.com/docs/rust/)

- closures are structs and can be eaten
- if you can't find a method on the container, look for a method on the iterator
- greeting.borrow().len() is fine since method calls will dereference implicitly

---
<!--ID:1689137981109-->
1. What output format options are there?
> ```rust
> {} calls display on an object
> {:?} calls debug on an object
> {:#?} pretty-print the debug formatting
> ```
<!--ID:1689137981111-->
1. What is the difference between :: and . ?
> . is used when you have a value on the left-hand-side. :: is used when you have a type or module.
>
> Or: . is for value member access, :: is for namespace member access.

---

# Infrastructure/Operations .......................................................................
```bash
# rustup.
rustup update  # upgrade to latest version.
rustup doc  # open offline documentation
rustup doc --std --cargo --core --reference --rust-by-example --test

# local documention of project crates:
cargo clean; cargo doc --open
# compile project and run
cargo run -- args

# run examples
cargo run --example confirm_simple
```
### Config Singleton and create config

---
<!--ID:1690041467827-->
1. How to create a mutable global Config singleton?
> - use the [config - Rust](https://docs.rs/config/latest/config/) crate: allows for stacked config
> - mutability via RW lock
> ```rust
> use config::Config;
> use lazy_static::lazy_static;
> use std::error::Error;
> use std::sync::RwLock;
>
> lazy_static! {
>     static ref SETTINGS: RwLock<Config> = RwLock::new(Config::default());
> }
> fn try_main() -> Result<(), Box<dyn Error>> {
>     // Set property
>     SETTINGS.write()?.set("property", 42)?;
>     // Get property
>     println!("property: {}", SETTINGS.read()?.get::<i32>("property")?);
>     Ok(())
> }
> fn main() {
>     try_main().unwrap();
> }
> ```

```rust
#[derive(Debug)]
pub struct Config {
    pub db_url: String,
    pub port: u16,
}

impl Config {
    fn new() -> Config {
        let db_url = env::var("DATABASE_URL").unwrap_or("../db_twbm.db".to_string());
        let port = env::var("PORT").unwrap_or("9999".to_string()).parse().expect("PORT must be a number");

        Config { db_url, port }
    }
}

// Create a global configuration singleton
lazy_static! {
    pub static ref CONFIG: Config = Config::new();
}
```


### dotenv
- `dotenv().ok()` should be called as early as possible in your program's execution, ideally at the beginning of your main function
- reads the environment variables from the .env file and loads them into the process's environment.
- These variables are then available to be accessed using the std::env module, just like any other environment variables.
- note that the dotenv crate will only load environment variables that are not already set in the current environment.

## Logging
[Logging in Rust - How to Get Started](https://www.shuttle.rs/blog/2023/09/20/logging-in-rust)
- the `log` crate only provides the API (facade), not implementations. You must pick a library that implements a logger.
- function name only via additonal crate, cannot be part of Formatter, must be passed in.
```rust
use log::{ info, warn, error, debug, };
fn main() {
    env_logger::init();  // RUST_LOG=debug
    debug!("Mary has a little lamb");
    error!("{}", "Its fleece was white as snow");
    info!("{:?}", "And every where that Mary went");
    warn!("{:#?}", "The lamb was sure to go");
}
```
